#!/bin/bash

# Sets up a new SIM
# usage: 
# ./setup.sh
# if we don't want to start the installed SIM and SAUS sevices provide
# ./setup.sh aws

set -e

RED='\033[0;31m'
GREEN='\033[0;32m'
CYAN="\033[0;36m"
ERROR='\033[41m'
NC='\033[0m'

SWAP_SIZE_GB=2
SWAPPINESS=5

trap 'last_command=$current_command; current_command=$BASH_COMMAND' DEBUG
trap 'catch' EXIT
catch() {
if [ $? -ne 0 ]; then
        printf "${ERROR}${last_command} command failed with exit code $?.${NC}\n"
        echo "ERROR: ${last_command}" >> setup.log 2>&1
fi
}

printf "\n\n${GREEN}Welcome to SIM Automated Installation Script.${NC}\n\n"
echo "================================" >> setup.log 2>&1
echo "SETUP STARTED" >> setup.log 2>&1

# Read user requirements
printf "\n${RED}Do you want this VM to serve recipes and templates? (y|n) ${NC}"
while true; do
    read wants_extra_server
    case $wants_extra_server in
        y ) break;;
        n ) break;;
        * ) echo "Please answer with y or n.";;
    esac
done

if [ "$wants_extra_server" = "y" ]; then
    printf "\n${RED}    Folder to use for recipes (under /var/www/html/): ${NC}"
    read recipes_folder
    printf "\n${RED}    Port to use for recipes: ${NC}"
    read recipes_port
    printf "\n${RED}    Folder to use for templates (under /var/www/html/): ${NC}"
    read templates_folder
    printf "\n${RED}    Port to use for templates: ${NC}"
    read templates_port
    echo "Wants extra server. $recipes_folder:$recipes_port, $templates_folder:$templates_port" >> setup.log 2>&1
fi

printf "\n${RED}Do you want to install Sunlight Automated Upgrade Service (SAUS)? (y|n) ${NC}"
while true; do
    read wants_saus
    case $wants_saus in
        y ) break;;
        n ) break;;
        * ) echo "Please answer with y or n.";;
    esac
done


# Identify Linux Distro
set +e
grep -i ubuntu /etc/os-release >> setup.log 2>&1
if [ $? -eq 0 ]; then
   web_server="apache2"
   distro="Ubuntu"
fi
grep -i "Red Hat Enterprise Linux" /etc/os-release >> setup.log 2>&1
if [ $? -eq 0 ]; then
   web_server="httpd"
   distro="RHEL"
fi
if [ "$distro" = "" ]; then
    printf "${ERROR}Could not iddentify a valid Linux Distro (Ubuntu or RHEL).${NC}"
    exit 1
fi
set -e
printf "${CYAN}Identified Linux Distro: $distro.${NC}\n"
echo "Identified Linux Distro: $distro." >> setup.log 2>&1

# Create swap space at /swapfile
if [[ $SWAP_SIZE_GB -ne 0 ]]; then
    echo "Creating swap space $i((SWAP_SIZE_G))B GB." >> setup.log 2>&1

    fallocate -l $((SWAP_SIZE_GB))G /swapfile >> setup.log 2>&1
    chmod 600 /swapfile >> setup.log 2>&1
    mkswap /swapfile >> setup.log 2>&1
    swapon /swapfile >> setup.log 2>&1
    cp /etc/fstab /etc/fstab.bak >> setup.log 2>&1
    echo "/swapfile none swap sw 0 0" | tee -a /etc/fstab >> setup.log 2>&1
    sysctl vm.swappiness=$((SWAPPINESS)) >> setup.log 2>&1
fi

# Check if packages are already installed
set +e
install_packages=y
if [ "$distro" = "Ubuntu" ]; then
    which apache2 >> setup.log 2>&1
    if [ $? -eq 0 ]; then
        which mysql >> setup.log 2>&1
         if [ $? -eq 0 ]; then
            which wget >> setup.log 2>&1
            if [ $? -eq 0 ]; then
                which etcd >> setup.log 2>&1
                if [ $? -eq 0 ]; then
                    install_packages=n
                fi
            fi
         fi
    fi
else
    which httpd>> setup.log 2>&1
    if [ $? -eq 0 ]; then
        which mysqld>> setup.log 2>&1
         if [ $? -eq 0 ]; then  
            which wget>> setup.log 2>&1
            if [ $? -eq 0 ]; then
                which etcd >> setup.log 2>&1
                if [ $? -eq 0 ]; then
                    install_packages=n
                fi
            fi
         fi
    fi
fi

echo "install_packages: $install_packages." >> setup.log 2>&1

has_certbot=n
which certbot >> setup.log 2>&1
if [ $? -eq 0 ]; then
    has_certbot=y
fi
echo "has_certbot: $has_certbot." >> setup.log 2>&1
set -e
if [ "$install_packages" = "n" ]; then
    printf "\n${RED}All necessary packages (web server, DB server and wget) are installed.\n"
    if [ "$has_certbot" = "n" ]; then
        printf "Certbot which is required for adding a secure domain is not installed.\n"
    fi
    printf "Do you want to proceed with the existing packages? (y|n) ${NC}"
    while true; do
        read proceed
        case $proceed in
            y ) break;;
            n ) break;;
            * ) echo "Please answer with y or n.";;
        esac
    done
else
    printf "\n${RED}Some of the required packages (web server, DB server, docker, etcd and wget) are not installed.\n"
    printf "Do you want to proceed with installing the required packages? (y|n) ${NC}"
    while true; do
        read proceed
        case $proceed in
            y ) break;;
            n ) break;;
            * ) echo "Please answer with y or n.";;
        esac
    done
fi

if [ "$proceed" = "n" ]; then
    printf "\n${ERROR}Please update your packages and then proceed. ${NC}\n"
    if [ "$install_packages" = "n" ]; then
         printf "${CYAN}The './cleanup.sh -r' script can be run to uninstall the packages. setup.sh will then reinstall them.${NC}\n"
    else
        if [ "$distro" = "Ubuntu" ]; then
             printf "${CYAN}The required pachages are apache2, mysql-server, docker, alpine/ansible docker image, wget , etcd and certbot.${NC}\n"
        else
             printf "${CYAN}The required pachages are httpd, mysqld, docker, alpine/ansible docker image, wget , etcd and certbot.${NC}\n"
        fi
    fi
    exit 0
fi

# Create default user
# If changed here there are other configuration that should be changed too
default_user="sim-user"
printf "${CYAN}Creating default user: $default_user.${NC}\n"
if [ "$distro" = "Ubuntu" ]; then
    adduser --gecos "" --disabled-password $default_user >> setup.log 2>&1
else
    adduser $default_user >> setup.log 2>&1
fi
echo "$default_user ALL=(ALL) NOPASSWD:ALL" >> /etc/sudoers
echo "Created user: $default_user." >> setup.log 2>&1

# Update and install prerequisites
if [ "$distro" = "Ubuntu" ]; then
    if [ "$install_packages" = "y" ]; then
        printf "${CYAN}Updating and installing required software...${NC}\n"
        apt-get -y update >> setup.log 2>&1
        printf "${CYAN}Still installing...${NC}\n"
        apt-get -y install apache2 >> setup.log 2>&1
        apt-get -y install mariadb-server mariadb-client >> setup.log 2>&1
        apt-get -y install wget >> setup.log 2>&1
        apt-get -y install certbot python3-certbot-apache >> setup.log 2>&1
        apt-get -y install zip >> setup.log 2>&1
        apt-get -y  install docker.io >> setup.log 2>&1
        #install etcd service
        ETCD_VER=v3.4.24
        GOOGLE_URL=https://storage.googleapis.com/etcd
        GITHUB_URL=https://github.com/etcd-io/etcd/releases/download
        DOWNLOAD_URL=${GOOGLE_URL} >> setup.log 2>&1

        rm -f /tmp/etcd-${ETCD_VER}-linux-amd64.tar.gz
        rm -rf /tmp/etcd-download-test && mkdir -p /tmp/etcd-download-test

        curl -L ${DOWNLOAD_URL}/${ETCD_VER}/etcd-${ETCD_VER}-linux-amd64.tar.gz -o /tmp/etcd-${ETCD_VER}-linux-amd64.tar.gz >> setup.log 2>&1
        tar xzvf /tmp/etcd-${ETCD_VER}-linux-amd64.tar.gz -C /tmp/etcd-download-test --strip-components=1 >> setup.log 2>&1
        rm -f /tmp/etcd-${ETCD_VER}-linux-amd64.tar.gz
        cp /tmp/etcd-download-test/etcd /usr/local/bin/
        cp /tmp/etcd-download-test/etcdctl /usr/local/bin/
        # Wait for docker to be up and running
        sleep 30
        docker pull docker.io/alpine/ansible:latest >> setup.log 2>&1
    fi
    systemctl enable mariadb >> setup.log 2>&1
    systemctl start mariadb >> setup.log 2>&1
else
    if [ "$install_packages" = "y" ]; then
        printf "${CYAN}Updating and installing required software...${NC}\n"
        dnf -y update >> setup.log 2>&1
        printf "${CYAN}Still installing...${NC}\n"
        dnf -y install httpd >> setup.log 2>&1
        dnf -y install mariadb-server mariadb-client >> setup.log 2>&1
        dnf -y install wget >> setup.log 2>&1
        dnf -y install https://dl.fedoraproject.org/pub/epel/epel-release-latest-8.noarch.rpm >> setup.log 2>&1
        dnf -y install certbot python3-certbot-apache >> setup.log 2>&1
        dnf -y install zip >> setup.log 2>&1
        dnf -y install docker >> setup.log 2>&1
        #install etcd service
        ETCD_VER=v3.4.24
        GOOGLE_URL=https://storage.googleapis.com/etcd
        GITHUB_URL=https://github.com/etcd-io/etcd/releases/download
        DOWNLOAD_URL=${GOOGLE_URL} >> setup.log 2>&1

        rm -f /tmp/etcd-${ETCD_VER}-linux-amd64.tar.gz
        rm -rf /tmp/etcd-download-test && mkdir -p /tmp/etcd-download-test

        curl -L ${DOWNLOAD_URL}/${ETCD_VER}/etcd-${ETCD_VER}-linux-amd64.tar.gz -o /tmp/etcd-${ETCD_VER}-linux-amd64.tar.gz >> setup.log 2>&1
        tar xzvf /tmp/etcd-${ETCD_VER}-linux-amd64.tar.gz -C /tmp/etcd-download-test --strip-components=1 >> setup.log 2>&1
        rm -f /tmp/etcd-${ETCD_VER}-linux-amd64.tar.gz
        cp /tmp/etcd-download-test/etcd /usr/local/bin/
        cp /tmp/etcd-download-test/etcdctl /usr/local/bin/
        # Wait for docker to be up and running
        sleep 30
        docker pull docker.io/alpine/ansible:latest >> setup.log 2>&1
    fi
    systemctl enable mariadb >> setup.log 2>&1
    systemctl start mariadb >> setup.log 2>&1
fi

printf "${CYAN}Setting Up MySQL.${NC}\n"
mysql_root_pass=$(cat /proc/sys/kernel/random/uuid | sed 's/[-]//g' | head -c 20;)
mysql_sim_pass=$(cat /proc/sys/kernel/random/uuid | sed 's/[-]//g' | head -c 20;)
mysql_saus_pass=$(cat /proc/sys/kernel/random/uuid | sed 's/[-]//g' | head -c 20;)

mysql -u root <<-EOF
DELETE FROM mysql.db WHERE Db='test' OR Db='test_%';
EOF

mysql -uroot -pasdasd <<-EOF
USE mysql;
ALTER USER 'root'@'localhost' IDENTIFIED BY '$mysql_root_pass';
FLUSH PRIVILEGES;
EOF

cp sim_api/.my.cnf /root >> setup.log 2>&1
sed -i "s/%ROOT_PASSWORD%/$mysql_root_pass/" /root/.my.cnf >> setup.log 2>&1

mysql -u root <<-EOF
USE mysql;
CREATE DATABASE sim2;
CREATE USER 'sim'@'localhost' IDENTIFIED BY '$mysql_sim_pass';
GRANT ALL PRIVILEGES ON sim2.* TO 'sim'@'localhost';
GRANT RELOAD ON *.* TO 'sim'@'localhost';
FLUSH PRIVILEGES;
EOF

if [ "$wants_saus" = "y" ]; then
mysql -u root <<-EOF
USE mysql;
CREATE DATABASE sausdb;
CREATE USER 'saus'@'localhost' IDENTIFIED BY '$mysql_saus_pass';
GRANT ALL PRIVILEGES ON sausdb.* TO 'saus'@'localhost';
GRANT RELOAD ON *.* TO 'saus'@'localhost';
FLUSH PRIVILEGES;
EOF
fi

# Turn off updates
printf "${CYAN}Disabling Automatic updates.${NC}\n"
if [ "$distro" = "Ubuntu" ]; then
    sed -i "s/1/0/" /etc/apt/apt.conf.d/20auto-upgrades  >> setup.log 2>&1 || true
else
    sed -i "s/apply_updates = yes/apply_updates = no/" /etc/dnf/automatic.conf >> setup.log 2>&1 || true
fi

# Update firewall 
if [ "$distro" = "Ubuntu" ]; then
    if ufw status | grep -q ' active'; then
        printf "${CYAN}Setting up ufw.${NC}\n"
        printf "${CYAN}    ufw is active. Adding required rules.${NC}\n"
        ufw allow "apache" >> setup.log 2>&1
        ufw allow "apache secure" >> setup.log 2>&1
        ufw allow ssh >> setup.log 2>&1
        ufw allow 7000:9999/tcp  >> setup.log 2>&1
        ufw allow 7000:9999/udp >> setup.log 2>&1
        ufw allow 12000:12999/tcp  >> setup.log 2>&1
        ufw allow 12000:12999/udp >> setup.log 2>&1
        ufw allow 15000:15999/tcp  >> setup.log 2>&1
        ufw allow 15000:15999/udp >> setup.log 2>&1
        if [ "$wants_extra_server" = "y" ]; then
            ufw allow recipes_port/tcp  >> setup.log 2>&1
            ufw allow recipes_port/udp >> setup.log 2>&1
            ufw allow templates_port/tcp  >> setup.log 2>&1
            ufw allow templates_port/udp >> setup.log 2>&1
        fi
        if [ "$wants_saus" = "y" ]; then
            ufw allow 23456/tcp  >> setup.log 2>&1
            ufw allow 23456/udp >> setup.log 2>&1
        fi
        ufw reload >> setup.log 2>&1
    fi
else
    set +e
    firewall-cmd >> setup.log 2>&1
    if [ $? -eq 0 ]; then
        if firewall-cmd --state | grep -q ' active'; then
            firewall-cmd --zone=public --permanent --add-service=ssh >> setup.log 2>&1
            firewall-cmd --zone=public --permanent --add-service=http >> setup.log 2>&1
            firewall-cmd --zone=public --permanent --add-service=https >> setup.log 2>&1
            firewall-cmd --permanent --zone=public --add-port=7000-9999/tcp >> setup.log 2>&1
            firewall-cmd --permanent --zone=public --add-port=7000-9999/udp >> setup.log 2>&1
            firewall-cmd --permanent --zone=public --add-port=12000-12999/tcp >> setup.log 2>&1
            firewall-cmd --permanent --zone=public --add-port=12000-12999/udp >> setup.log 2>&1
            firewall-cmd --permanent --zone=public --add-port=15000-15999/tcp >> setup.log 2>&1
            firewall-cmd --permanent --zone=public --add-port=15000-15999/udp >> setup.log 2>&1
            if [ "$wants_extra_server" = "y" ]; then
                firewall-cmd --permanent --zone=public --add-port=recipes_port/tcp >> setup.log 2>&1
                firewall-cmd --permanent --zone=public --add-port=recipes_port/udp >> setup.log 2>&1
                firewall-cmd --permanent --zone=public --add-port=templates_port/tcp >> setup.log 2>&1
                firewall-cmd --permanent --zone=public --add-port=templates_port/udp >> setup.log 2>&1
            fi
            if [ "$wants_saus" = "y" ]; then
                firewall-cmd --permanent --zone=public --add-port=23456/tcp >> setup.log 2>&1
                firewall-cmd --permanent --zone=public --add-port=23456/udp >> setup.log 2>&1
            fi
            firewall-cmd --reload >> setup.log 2>&1
        fi
    fi
    set -e
fi

# Setup Web Server
printf "${CYAN}Setting up Web server.${NC}\n"
printf "${CYAN}    Setting up Web server modules.${NC}\n"
if [ "$distro" = "Ubuntu" ]; then
    a2enmod proxy >> setup.log 2>&1
    a2enmod proxy_http >> setup.log 2>&1
    a2enmod rewrite >> setup.log 2>&1
    a2enmod deflate >> setup.log 2>&1
    a2enmod headers >> setup.log 2>&1
    a2enmod proxy_html >> setup.log 2>&1
    a2enmod proxy_balancer >> setup.log 2>&1
    a2enmod proxy_connect >> setup.log 2>&1
    a2enmod proxy_wstunnel >> setup.log 2>&1
    a2enmod ssl >> setup.log 2>&1
else
    echo "LoadModule proxy_module modules/mod_proxy.so" >> /etc/httpd/conf/httpd.conf
    echo "LoadModule proxy_connect_module modules/mod_proxy_connect.so" >> /etc/httpd/conf/httpd.conf
    echo "LoadModule proxy_balancer_module modules/mod_proxy_balancer.so" >> /etc/httpd/conf/httpd.conf
    echo "LoadModule proxy_http_module modules/mod_proxy_http.so" >> /etc/httpd/conf/httpd.conf
    echo "LoadModule rewrite_module modules/mod_rewrite.so" >> /etc/httpd/conf/httpd.conf
    echo "LoadModule deflate_module modules/mod_deflate.so" >> /etc/httpd/conf/httpd.conf
    echo "LoadModule headers_module modules/mod_headers.so" >> /etc/httpd/conf/httpd.conf
    echo "LoadModule proxy_wstunnel_module modules/mod_proxy_wstunnel.so" >> /etc/httpd/conf/httpd.conf
    echo "IncludeOptional sites-enabled/*.conf" >> /etc/httpd/conf/httpd.conf
    sed -i "s/Listen .*//g"  /etc/httpd/conf/httpd.conf
    echo "Listen 80" >> /etc/httpd/conf/httpd.conf
    echo "Listen 443" >> /etc/httpd/conf/httpd.conf
    if [ "$wants_extra_server" = "y" ]; then
        echo "Listen $recipes_port" >> /etc/httpd/conf/httpd.conf
        echo "Listen $templates_port" >> /etc/httpd/conf/httpd.conf
    fi
fi

printf "${CYAN}    Setting up Web Server sites.${NC}\n"
rm -r /etc/$web_server/sites-available/* >> setup.log 2>&1 || true
rm -r /etc/$web_server/sites-enabled/*>> setup.log 2>&1 || true
rm -r /var/www/html/* >> setup.log 2>&1 || true
mkdir -p /etc/$web_server/sites-available /etc/$web_server/sites-enabled >> setup.log 2>&1
cp web-server/confs/* /etc/$web_server/sites-available/ >> setup.log 2>&1
if [ "$wants_extra_server" = "y" ]; then
    mkdir -p /var/www/html/$recipes_folder >> setup.log 2>&1
    mkdir -p /var/www/html/$templates_folder >> setup.log 2>&1
    cp web-server/index.html /var/www/html/$recipes_folder >> setup.log 2>&1
    sed -i "s/%WHAT%/Recipes/" /var/www/html/$recipes_folder/index.html >> setup.log 2>&1
    cp web-server/index.html /var/www/html/$templates_folder >> setup.log 2>&1
    sed -i "s/%WHAT%/Templates/" /var/www/html/$templates_folder/index.html >> setup.log 2>&1
    if [ "$distro" = "RHEL" ]; then
        sed -i "s/Listen .*//g" /etc/$web_server/sites-available/recipes.conf >> setup.log 2>&1
        sed -i "s/Listen .*//g" /etc/$web_server/sites-available/templates.conf >> setup.log 2>&1
    fi
    sed -i "s/%PORT%/$recipes_port/" /etc/$web_server/sites-available/recipes.conf >> setup.log 2>&1
    sed -i "s/%PATH%/$recipes_folder/" /etc/$web_server/sites-available/recipes.conf >> setup.log 2>&1
    sed -i "s/%PORT%/$templates_port/" /etc/$web_server/sites-available/templates.conf >> setup.log 2>&1
    sed -i "s/%PATH%/$templates_folder/" /etc/$web_server/sites-available/templates.conf >> setup.log 2>&1

    ln -s /etc/$web_server/sites-available/recipes.conf /etc/$web_server/sites-enabled/recipes.conf >> setup.log 2>&1
    ln -s /etc/$web_server/sites-available/templates.conf /etc/$web_server/sites-enabled/templates.conf >> setup.log 2>&1
    if [ "$distro" = "RHEL" ]; then
        semanage port -a -t http_port_t -p tcp $recipes_port >> setup.log 2>&1 || true
        semanage port -a -t http_port_t -p udp $recipes_port >> setup.log 2>&1 || true
        semanage port -a -t http_port_t -p tcp $templates_port >> setup.log 2>&1 || true
        semanage port -a -t http_port_t -p udp $templates_port >> setup.log 2>&1 || true
    fi
fi

if [ "$distro" = "Ubuntu" ]; then
    rm /etc/$web_server/sites-available/000-default.conf >> setup.log 2>&1 || true
else
    rm /etc/httpd/conf.d/*.conf >> setup.log 2>&1 || true
    /usr/sbin/setsebool -P httpd_can_network_connect 1 >> setup.log 2>&1
fi

ln -s /etc/$web_server/sites-available/sim2.conf /etc/$web_server/sites-enabled/sim2.conf >> setup.log 2>&1

printf "${CYAN}    Restarting Web Server.${NC}\n"
systemctl restart $web_server >> setup.log 2>&1

# Setup sim UI
cp -r sim_ui /var/www/html >> setup.log 2>&1
mv /var/www/html/sim_ui /var/www/html/sim2 >> setup.log 2>&1

# Setup sim API & conf & service & logrotate
printf "${CYAN}Setting up SIM API.${NC}\n"

# Stop service if it runs
service sim stop >> setup.log 2>&1 || true
service sim disable >> setup.log 2>&1 || true
service saus disable >> setup.log 2>&1 || true
service saus stop >> setup.log 2>&1 || true
rm /usr/local/bin/saus-client >> setup.log 2>&1 || true

printf "${CYAN}Moving SIM API files.${NC}\n"
mkdir -p /usr/local/bin/ >> setup.log 2>&1
cp sim_api/sim_api /usr/local/bin/ >> setup.log 2>&1
chmod +x /usr/local/bin/sim_api >> setup.log 2>&1
chown root:root /usr/local/bin/sim_api  >> setup.log 2>&1
cp -r sim_api/scripts /home/$default_user >> setup.log 2>&1
chmod 755 /home/$default_user/scripts/* >> setup.log 2>&1
chown $default_user:$default_user /home/$default_user/scripts/* >> setup.log 2>&1
mkdir -p /home/$default_user/.ssh >> setup.log 2>&1
chown $default_user:$default_user /home/$default_user/.ssh >> setup.log 2>&1
chmod 700 /home/$default_user/.ssh >> setup.log 2>&1
cp -r sim_api/sshKeys /home/$default_user/ >> setup.log 2>&1


chmod 600 /home/$default_user/sshKeys/authorized_keys >> setup.log 2>&1
chown $default_user:$default_user /home/$default_user/sshKeys/authorized_keys >> setup.log 2>&1
chmod 600 /home/$default_user/sshKeys/id_rsa_sim >> setup.log 2>&1
chown $default_user:$default_user /home/$default_user/sshKeys/id_rsa_sim >> setup.log 2>&1
chmod 600 /home/$default_user/sshKeys/id_rsa_sim.pub.pem >> setup.log 2>&1
chown $default_user:$default_user /home/$default_user/sshKeys/id_rsa_sim.pub.pem >> setup.log 2>&1
chmod 600 /home/$default_user/sshKeys/manager_vm.pem >> setup.log 2>&1
chown $default_user:$default_user /home/$default_user/sshKeys/manager_vm.pem >> setup.log 2>&1
chmod 644 /home/$default_user/sshKeys/manager_vm.pub >> setup.log 2>&1
chown $default_user:$default_user /home/$default_user/sshKeys/manager_vm.pub >> setup.log 2>&1

touch /home/$default_user/.ssh/authorized_keys >> setup.log 2>&1
chmod 600 /home/$default_user/.ssh/authorized_keys >> setup.log 2>&1
chown $default_user:$default_user /home/$default_user/.ssh/authorized_keys >> setup.log 2>&1

printf "${CYAN}Creating SIM API service.${NC}\n"
cp sim_api/sim.service /lib/systemd/system/ >> setup.log 2>&1
ln -s /lib/systemd/system/sim.service /etc/systemd/system/ >> setup.log 2>&1


printf "${CYAN}Creating etcd service.${NC}\n"
cp etcd/etcd.service /lib/systemd/system/ >> setup.log 2>&1
ln -s /lib/systemd/system/etcd.service /etc/systemd/system/ >> setup.log 2>&1

printf "${CYAN}Applying logrotate to SIM API logs.${NC}\n"
cp sim_api/sim_api_logrotate /etc/logrotate.d/ >> setup.log 2>&1

printf "${CYAN}Adding SIM API configuration file.${NC}\n"
cp sim_api/sim-api.conf /etc/ >> setup.log 2>&1
mysql_pass_conf="mysql_password=\"$mysql_sim_pass\""
sed -i "s/#mysql_password=\"\"/$mysql_pass_conf/" /etc/sim-api.conf >> setup.log 2>&1
home_append="home_folder=\"/home/$default_user\""
echo $home_append >> /etc/sim-api.conf >> setup.log 2>&1

# Prompt user to select network device
networks=`ls /sys/class/net` >> setup.log 2>&1
for value in "${networks[@]}"
do
    [[ $value != "lo" ]] && networks_array+=($value)
done

i=1
printf "${RED}Select a network interface to use:${NC}\n"
for network in "${networks_array[@]}"
do
    printf "${RED}$i. ${networks_array[$((i-1))]} ${NC}\n"
    i=$((i+1))
done

select_network=1000
while [ "$select_network" -gt $((i-1)) ]
do
    printf "\n${RED}Select 1-$((i-1)): ${NC}"
    read select_network
done

printf "\n${CYAN}Selected network: ${networks_array[$((select_network-1))]}.${CYAN}\n"
network_append="eth_id=\"${networks_array[$((select_network-1))]}\""
sed -i "s/#eth_id=\"eth0\"/$network_append/" /etc/sim-api.conf >> setup.log 2>&1
printf "${CYAN}Can be later altered from SIM configuration file at /etc/sim.conf${NC}\n"

# Setup SAUS
if [ "$wants_saus" = "y" ]; then
    printf "${CYAN}Setting up SAUS..${NC}\n"
    mkdir -p /etc/saus/ >> setup.log 2>&1
    mkdir -p /usr/local/bin/ >> setup.log 2>&1
    cp saus/saus-client /usr/local/bin/ >> setup.log 2>&1
    chmod +x /usr/local/bin/saus-client >> setup.log 2>&1
    chown root:root /usr/local/bin/saus-client  >> setup.log 2>&1
    mkdir -p /downloads >> setup.log 2>&1
    chown $default_user:$default_user /downloads >> setup.log 2>&1
    cp saus/.version /home/$default_user/ >> setup.log 2>&1
    cp saus/saus.conf /etc/saus/ >> setup.log 2>&1
    sed -i "s/%SAUS_PASSWORD%/$mysql_saus_pass/" /etc/saus/saus.conf  >> setup.log 2>&1
    cp saus/saus_logrotate /etc/logrotate.d/ >> setup.log 2>&1
    cp saus/saus.service /lib/systemd/system/ >> setup.log 2>&1
    ln -s /lib/systemd/system/saus.service /etc/systemd/system/ >> setup.log 2>&1
fi

systemctl daemon-reload >> setup.log 2>&1
# Enable and start services
if [[ $# -ne 0 && "$1" == "aws" ]]; then
    printf "${CYAN}Enabling SIM API service.${NC}\n"
    systemctl enable sim >> setup.log 2>&1
     if [ "$wants_saus" = "y" ]; then
        printf "${CYAN}Enabling SAUS service.${NC}\n"
        systemctl enable saus >> setup.log 2>&1
    fi
    printf "${CYAN}Enabling etcd service.${NC}\n"
    systemctl enable etcd >> setup.log 2>&1
else
    printf "${CYAN}Enabling and starting SIM API service.${NC}\n"
    service sim start >> setup.log 2>&1
    systemctl enable sim >> setup.log 2>&1
    if [ "$wants_saus" = "y" ]; then
        printf "${CYAN}Enabling and starting SAUS service.${NC}\n"
        service saus start >> setup.log 2>&1
        systemctl enable saus >> setup.log 2>&1
    fi
    printf "${CYAN}Enabling and starting etcd service.${NC}\n"
    service etcd start >> setup.log 2>&1
    systemctl enable etcd >> setup.log 2>&1
fi

printf "\n${GREEN}SIM has been successfully installed.\n"
if [ "$wants_saus" = "y" ]; then
    printf "SAUS has been successfully installed. SIM will get latest updates automatically.${NC}\n"
fi

if [ "$wants_extra_server" = "y" ]; then
    printf "\n${CYAN}##########################################################################################################################\n"
    printf "Proceed setting up Recipes server using: \n"
    printf "Recipes URL: http://{SIM_IP|SIM_DOMAIN}:$recipes_port\n"
    printf "Recipes Path: /var/www/html/$recipes_folder/\n"
   
    printf "##########################################################################################################################\n"
    printf "Proceed setting up Templates server using: \n"
    printf "Templates URL: http://{SIM_IP|SIM_DOMAIN}:$templates_port\n"
    printf "Templates Path: /var/www/html/$templates_folder/\n"

    printf "##########################################################################################################################\n"
    printf "\nSIM Configuration:\n\n"
    printf "Configuration 'recipe_url' of SIM should point to a .tar.gz file within the\n"
    printf "recipes Path and should accommodate the tar.gz file that contains :\n"
    printf "   * A 'recipes.json' file that contains a list of recipes\n"
    printf "     (name, version, description, category and filename).\n"
    printf "   * A .json for each of the recipes with the filename given at the recipes.json\n"
    printf "     file with the infrastructure and configuration details in json format.\n"
    printf "e.g. recipe_url='http://{SIM_IP|SIM_DOMAIN}:6001/LATEST/sim_templates/sim_templates.tar.gz'\n"

    printf "\nConfiguration 'default_marketplace_repo_url' of SIM should point to a .json file.\n"
    printf "Templates Path should accommodate the json file that contains a list of templates\n"
    printf "with all the required information (os_distro, name, filename, os_version, architecture etc).\n"
    printf "At the same directory of .json file there should be all the required images in .tar.gz format.\n"
    printf "e.g. default_marketplace_repo_url='http://{SIM_IP|SIM_DOMAIN}:6002/marketplace/templates.json'\n"
    
    printf "\nA restart is required for SIM after the configuration update.\n"
    printf "SIM configuration under '/etc/sim-api.conf'. SIM restart with 'service restart sim-api'.\n"

    printf "\n##########################################################################################################################\n"
    printf "\nClusters Configuration:\n\n"
    printf "Configuration 'recipes_url' and 'recipes_version' of each cluster should accommodate a .tar.gz file of the ansible playbook\n"
    printf "files alongside a .hash file that contains the hash of the .tar.gz file. Both should have the filename from the json file.\n"
    printf "Configurations 'recipes_url' and 'recipes_version' should construct a URL with the form of recipes_url/recipes_version\n"
    printf "where the required files will be located.\n"
    printf "e.g. recipes_url='http://{SIM_IP|SIM_DOMAIN}:6001/recipes.sunlight.io/'\n"
    printf "e.g. recipes_version='LATEST'\n"
    
    printf "\nConfiguration 'templates_link' of each cluster should accommodate a json file\n"
    printf "with the metatadata os each image that is hosted.\n"
    printf "e.g. templates_link='http://{SIM_IP|SIM_DOMAIN}:6002/templates/templates.json'\n"

    printf "\nConfiguration 'template_images' of each cluster should accommodate all the images files\n"
    printf "in a .tar.gz. format alongside the .hash file of each image.\n"
    printf "e.g. template_images='http://{SIM_IP|SIM_DOMAIN}:6002/templates/'\n\n"
  
    printf "A restart is required for the virtupian-api (at each cluster) after the configuration update.\n"
    printf "Virtupian (cluster) configuration under '/etc/virtupian-api/virtupian-api.conf'. Virtupian restart with '/etc/init.d/virtupian-api restart'.\n"

    printf "\n##########################################################################################################################${NC}\n"
fi

printf "\n${GREEN}You can now visit your SIM's URL or IP and login according to the documentation.\n"

# All Finished
printf "\n\n"
printf " - ALL DONE - \n"
printf "${NC}\n\n"

echo "SETUP COMPLETED" >> setup.log 2>&1
echo "================================" >> setup.log 2>&1
